# -*- coding: utf-8 -*-

# number of item can be shown in the UI
SHOW_ITEMS_LIMIT = 10

# how many UP/DOWN key pressed when scrolling the UI.
SCROLL_SPEED = 5

DEFAULT_TITLE = "NO TITLE"
DEFAULT_SUBTITLE = "NO SUBTITLE"
